/*
   Copyright 2005 (C) Jean-Baptiste Note

   This file is part of islsm.

   islsm is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   islsm is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with islsm; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <getopt.h>
#include <sys/ioctl.h>
#include <stdlib.h>

/* for strncpy */
#include <string.h>

/* for open */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* for close */
#include <unistd.h>

/* this program is a simple interface to send ioctls to the softmac
 * devices */

/* ioctl interface -- from kernel module */
#include <islsm_smioctl.h>

int
main(int argc, char *argv[])
{
	int fd, err;

	static int verbose;
	static int cmd;

#define MAX_LENGTH 1500
	unsigned length = 0x70;
	unsigned long arg = 0;

	struct islsm_reg reg_arg = { 0, 0 };

	char *device = "/dev/islsm0";

	static const char shortopts[] = "";

	static struct option longopts[] = {
		{"verbose", no_argument, &verbose, 1},
		/* device to use */
		{"dev", required_argument, 0, 'd'},
		/* arguments */
		{"addr", required_argument, 0, 'a'},
		{"value", required_argument, 0, 'v'},
		{"length", required_argument, 0, 'l'},
		/* commands */
		{"rreg", no_argument, &cmd, ISLSM_IOCGREG},
		{"rmem", no_argument, &cmd, ISLSM_IOCGMEM},
		{"wreg", no_argument, &cmd, ISLSM_IOCSREG},
		{"wmem", no_argument, &cmd, ISLSM_IOCSMEM},
		{"bootrom", no_argument, &cmd, ISLSM_IOCBOOTROM},
		{"uploadfw", no_argument, &cmd, ISLSM_IOCLOADFW},
		{"sendping", no_argument, &cmd, ISLSM_IOCSENDPING},
		{0, 0, 0, 0},
	};

	/* getopt processing loop */
	while (1) {
		int c;
		/* getopt_long stores the option index here. */
		int option_index = 0;

		c = getopt_long(argc, argv, shortopts, longopts, &option_index);

		/* Detect the end of the options. */
		if (c == -1)
			break;

		switch (c) {
		case 0:
			if (_IOC_NR(longopts[option_index].val) &
			    ISLSM_IOC_ARGTYPE_REG)
				arg = (unsigned long)&reg_arg;
			break;

		case 'd':
			device = optarg;
			break;
		case 'a':
			reg_arg.addr = (uint32_t)atoll(optarg);
			break;
		case 'v':
			reg_arg.val = (uint32_t)atoll(optarg);
			break;
		case 'l':
			length = (unsigned)atoi(optarg);
			if (length > MAX_LENGTH)
				exit(1);
			break;
		case '?':
			/* getopt_long already printed an error message. */
			/* abort, we don't want to fiddle with hardware
			   in case of error */
			return -1;
			break;

		default:
			abort();
		}
	}

	/* open device */
	if (verbose) {
		fprintf(stderr, "Opening device %s\n", device);
	}

	fd = open(device, O_RDWR);
	if (fd == -1) {
		perror("opening device ");
		return -1;
	}
	/* do ioctl */
	if (verbose) {
		fprintf(stderr, "Doing ioctl %i\n", cmd);
	}
	err = ioctl(fd, cmd, arg);
	if (err < 0) {
		perror("");
		return -err;
	}
	/* close */
	close(fd);

	/* prints the results */
	if (_IOC_NR(cmd) & ISLSM_IOC_ARGTYPE_REG)
		printf("addr %x, val %08x\n", reg_arg.addr, reg_arg.val);

	exit(0);
}
