/*
   Copyright 2005 Jean-Baptiste Note

   This file is part of islsm.

   islsm is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   islsm is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with islsm; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <unistd.h>

//#define VERBOSE

static void pack(char *src, unsigned src_length, FILE * out);
static int
 find_best_string(char *src, unsigned length, unsigned *offset, unsigned *len);

static void
pack(char *src, unsigned src_length, FILE * out)
{
	int total_written = 0;
	char *saved_src = src;
	unsigned saved_src_length = src_length;

	/* while we don't wrap */
	while (src_length <= saved_src_length) {
		/* contains the mask byte + data bytes */
		char buffer[1 + 2 * 8];
		unsigned buffer_written = 1;
		int i;

		/* note : we do not guarantee anything about the
		 * padding, the requester has to take care of it */
		for (i = 7; i >= 0; i--) {
			/* length of the best match */
			unsigned len = 0;
			/* 12 bits address, taken relative to the position where
			 * we intend to start writing the byte. Please
			 * note that as zero is not a possible value here,
			 * they're losing one bit on information */
			unsigned offset = 0;

			if (src_length > saved_src_length) {
				/* pad with zeroes. This should not be
				   seen by the decompressor anyways */
				buffer[0] &= ~(1 << i);
				buffer[buffer_written++] = 0;
				len = 1;
			} else {
				/* we may find a matching string */

				find_best_string(src, src - saved_src,
						 &offset, &len);

				if (len < 3) {
					/* clear bit in mask */
					buffer[0] &= ~(1 << i);
					buffer[buffer_written++] = *src;
					len = 1;
				} else {	/* compress string */

					/* find_best_string returns a length
					   between 3 and 18, and offset on 12 bits */
					buffer[0] |= (1 << i);
					buffer[buffer_written] = (len - 3) << 4;
					buffer[buffer_written++] |=
					    (offset & 0x0f00) >> 8;
					buffer[buffer_written++] =
					    (offset & 0xff);
				}
			}

			src += len;
			src_length -= len;
#ifdef VERBOSE
			fprintf(stderr,
				"Treated %i bytes of input data, %i bytes remaining\n",
				src - saved_src, src_length);
#endif				/* VERBOSE */
		}

		/* write the data */
		for (i = 0; i < buffer_written; i++)
			putc(buffer[i], out);

		total_written += buffer_written;
	}

	/* write the rest */
	fprintf(stderr, "%i bytes written\n", total_written);
	return;
}

/* use a modified KMP to do the
   compression should yield better perfs */
/* @src : the address of the character to be written
   @length : the length already written [src_start, src[
   @pos : the offset relative to src
   @len : the length of the matching string
*/
#define MAX_MATCH_LEN ((1<<4) - 1) + 3
#define MAX_OFFSET_ABS ((1<<12) - 1)
static int
find_best_string(char *src, unsigned length, unsigned *offset, unsigned *len)
{
	unsigned curoffset;
	*len = 0;

	// FIXME : we may read farther than src allows us. Fix this with
	// another parameter
	for (curoffset = 1;
	     curoffset < length && curoffset < MAX_OFFSET_ABS; curoffset++) {
		unsigned curlen = 0;
		char *base_src = src - curoffset;
		/* determine the max matching length */
		for (curlen = 0;
		     curlen < MAX_MATCH_LEN && (base_src + curlen) < src;
		     curlen++)
			if (base_src[curlen] != src[curlen])
				break;
		if (curlen > *len) {
			*offset = curoffset;
			*len = curlen;
			if (curlen == MAX_MATCH_LEN) {
				/* got an absolute best match, don't go further
				   down */
				goto end;
			}
		}
	}

      end:
#ifdef VERBOSE
	fprintf(stderr, "Best match @%p of length %i, offset %i\n", src,
		*len, *offset);
#endif				/* VERBOSE */
	return 0;
}

/* void pack_intelligent() */

char src[1024 * 1024];
char dst[1024 * 1024];

/* beware, we add garbage at the end of the compressed thing. Not a
 * problem it seems */

int
main()
{
	int c;
	int unpacked_len = 0;

	while ((c = getc(stdin)) != EOF) {
		src[unpacked_len++] = c;
		if (unpacked_len > sizeof(src))
			break;
	}

	fprintf(stderr, "%i bytes read, compressing\n", unpacked_len);

	pack(src, unpacked_len, stdout);
	return 0;
}
