/*
   Copyright 2005 Denis Vlasenko

   This file is part of islsm.

   islsm is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   islsm is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with islsm; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

void unpack(char *src, char *dst, char *dst_end);

void
unpack(char *r2, char *r1, char *r0)
{
	uint32_t r3, r4, r5;
	char *r6;
	int carry;
	char *r2_saved = r2;

      L198:
	r3 = (*r2++ << 24) | 0x800000;
      L1a4:
	carry = r3 & 0x80000000;
	r3 <<= 1;
	if (!r3)
		goto L198;

	r4 = (unsigned char)*r2++;

	if (!carry) {
		*r1++ = r4;
	} else {		// unpack a substring
		r5 = (r4 >> 4) + 3;
		r6 = r1 - ((r4 & 0xf) << 8) - (unsigned char)*r2++;
		do {
			*r1++ = *r6++;
		} while (--r5);
	}			// end_else

	if (r1 < r0)
		goto L1a4;	// continue to analyze bitmask
	fprintf(stderr, "Consumed bytes: %d (0x%x)\n", r2 - r2_saved,
		r2 - r2_saved);
}

char src[1024 * 1024];
char dst[1024 * 1024];

int
main(int argc, char *argv[])
{
	int r;
	int unpacked_len;

	if (argc < 2) {
		fprintf(stderr,
			"you must provide length for correct unpacking\n");
		return -1;
	}

	r = read(0, src, sizeof(src));
	if (r < 0) {
		perror("read");
		return 1;
	}
//      0x0001d928 for 1.0.4.3
	unpacked_len = atoi(argv[1]);
	unpack(src, dst, dst + unpacked_len);
	write(1, dst, unpacked_len);
	return 0;
}
