/*
 *  exported-global - test exported relay file ops with a global buffer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Copyright (C) 2005 - Tom Zanussi (zanussi@us.ibm.com), IBM Corp
 *
 * This example creates a global relay file in the debugfs filesystem
 * and uses read(2) to read from it.  It amounts to pretty much the
 * simplest possible usage of relayfs for ad hoc kernel logging.
 * Relayfs itself should be insmod'ed or configured in, but doesn't
 * need to be mounted.
 *
 * Usage:
 *
 * modprobe relayfs
 * mount -t debugfs debugfs /debug
 * ./exported-global debug_file
 *
 * captured output will appear on stdout
 *
 */
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>  /* fprintf */
#include <unistd.h> /* write */
#include <string.h> /* strerror */

static void process_data(const char *buf, int len)
{
#if 0
	fprintf(stderr,"read %08u bytes\n", len);
#endif
	/* Write to stdout for now, but we could do so much more... */
	write(1, buf, len);
}

int main(int argc, char **argv)
{
	char buf[4096];
	int rc, relay_file;
	char *relay_file_name = argv[1];

	if (argc < 2) {
		fprintf(stderr,"usage: %s relay_file\n", argv[0]);
		return -1;
	}

	relay_file = open(relay_file_name, O_RDONLY | O_NONBLOCK);
	if (relay_file < 0) {
		fprintf(stderr,"Couldn't open relay file %s: errcode = %s\n",
		        relay_file_name, strerror(errno));
		return -1;
	}

	do {
		rc = read(relay_file, buf, 4096);
		if (rc < 0) {
			perror("read");
			continue;
		} else if (rc > 0) {
			process_data(buf, rc);
			continue;
		}
		usleep(1000);
	} while (1);

	close(relay_file);
}

